﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using ToDo.MvcApp.Models;

namespace ToDo.MvcApp.Controllers
{
    public class TasksController : ApiController
    {
        // Per questa demo viene istanziato il repository direttamente nel controlle
        // ma in produzione è meglio evitare di farlo, utilizzando il Web API Dependency Resolver
        static readonly IToDoRepository repository = new ToDoRepository();

        // Get a list of all tasks: /api/tasks
        // The method name starts with "Get", so by convention it maps to GET requests.
        // Also, because the method has no parameters, it maps to a URI that does not contain an "id" segment in the path.
        public IEnumerable<Task> GetAllTasks()
        {
            return repository.GetAll();
        }

        // Get a task by ID: /api/tasks/id
        // This method name also starts with "Get", but the method has a parameter named id.
        // This parameter is mapped to the "id" segment of the URI path.
        // The ASP.NET Web API framework automatically converts the ID to the correct data type (int) for the parameter. 
        public Task GetTask(int id)
        {
            Task task = repository.Get(id);
            if (task == null)
            {
                throw new HttpResponseException(HttpStatusCode.NotFound);
            }
            return task;
        }

        // Get a task by category: /api/tasks?category=category 
        // If the request URI has a query string, Web API tries to match the query parameters to parameters on the controller method.
        // Therefore, a URI of the form "api/products?category=category" will map to this method.
        public IEnumerable<Task> GetTasksByCategory(string category)
        {
            return repository.GetAll().Where(
                p => string.Equals(p.Category, category, StringComparison.OrdinalIgnoreCase));
        }

        // Post a new task.
        // This example does not validate the task. For information about model validation, see Model Validation in ASP.NET Web API.
        // Response code: By default, the Web API framework sets the response status code to 200 (OK).
        // But according to the HTTP/1.1 protocol, when a POST request results in the creation of a resource,
        // the server should reply with status 201 (Created).
        // Location: When the server creates a resource, it should include the URI of the new resource in the Location header of the response.
        public HttpResponseMessage PostTask(Task task)
        {
            task = repository.Add(task);
            var response = Request.CreateResponse<Task>(HttpStatusCode.Created, task);

            string uri = Url.Link("DefaultApi", new { id = task.Id });
            response.Headers.Location = new Uri(uri);
            return response;
        }

        // Put new values to a task.
        // This example does not validate the task. For information about model validation, see Model Validation in ASP.NET Web API.
        // The method name starts with "Put...", so Web API matches it to PUT requests.
        // The method takes two parameters, the product ID and the updated product.
        // The  id parameter is taken from the URI path, and the product parameter is deserialized from the request body.
        // By default, the ASP.NET Web API framework takes simple parameter types from the route and complex types from the request body.
        public void PutTask(int id, Task task)
        {
            task.Id = id;
            if (!repository.Update(task))
            {
                throw new HttpResponseException(HttpStatusCode.NotFound);
            }
        }

        // Delete a task
        // If a DELETE request succeeds, it can return:
        // - status 200 (OK) with an entity-body that describes the status;
        // - status 202 (Accepted) if the deletion is still pending;
        // - status 204 (No Content) with no entity body.
        // In this case, the DeleteProduct method has a void return type,
        // so ASP.NET Web API automatically translates this into status code 204 (No Content).
        public void DeleteTask(int id)
        {
            Task task = repository.Get(id);
            if (task == null)
            {
                throw new HttpResponseException(HttpStatusCode.NotFound);
            }
            repository.Remove(id);
        }

    }
}
